/** @file CanvasExtend.h
 *
 * @author marco corvi
 * @brief extedn directions
 */
#ifndef CANVAS_EXTEND_H
#define CANVAS_EXTEND_H

#define DIR_LEFT -1
#define DIR_RIGHT 1
#define DIR_VERT  0


/** difference between two angles in [0, 360)
 * @param a1 first angle
 * @param a2 second angle
 * @return the difference angle: a1 - a2
 */
double angle_difference( double a1, double a2 );

/** A canvas extend is a value of "extend" associated to an interval
 * of angular directions. As the CanvasExtends are used only in arrays,
 * ordered around the circle, it is enough to record only the start angle
 * of the interval, the end angle being te start angle of the next
 * CanvasExtend in the array.
 */
struct CanvasExtend
{
  int dir;      //!< extend direction: -1 left, 0 vertical, +1 right 
  double angle; //!< angle where this extend begins

  CanvasExtend( int d, double a )
    : dir( d )
    , angle( a )
  { }

  /** operator less
   * @param ce   the other CanvasExtend
   * @return true if this angle is less than the other CanvasExtend angle
   */
  bool operator < ( const CanvasExtend & ce ) { return angle < ce.angle; }

  double operator-( const CanvasExtend & ce ) { return angle_difference( angle, ce.angle ); }

  double operator-( double a ) { return angle_difference( angle, a ); }

};

/** Interpolate between two extends
 * @param e1 first extend
 * @param e2 second extend
 * @param alpha angle to interpolate
 * @return interpolated extend (between -1 (left) and +1 (right))
 */ 
double interpolate_extend( const CanvasExtend & e1, const CanvasExtend & e2, double alpha );

#endif

