/** @file CanvasUndo.h
 *
 * @author marco corvi
 * @date dec 2009
 *
 * @brief canvas undo list
 * --------------------------------------------------------
 *  Copyright This sowftare is distributed under GPL-3.0 or later
 *  See the file COPYING.
 */
#ifndef CANVAS_UNDO_H
#define CANVAS_UNDO_H

#define UNDO_POINT     0
#define UNDO_LINEPOINT 1
#define UNDO_ENDLINE   2
#define UNDO_CLOSELINE 3

/** undo struct
 * LIFO list of undo-commands
 * the undo are inserted at the head of the list and removed from the head
 */
struct CanvasUndo
{
  int command;              //!< commandtype: 0 point, 1 line-point, 2 end-line
  struct CanvasUndo * next; //!< link
 
  public:
    /** cstr
     * @param cmd   command type
     * @param n     next canvas-undo
     *
     * @note the cstr puts the new CanvasUndo at the head of th list of undo's
     */
    CanvasUndo( int cmd, CanvasUndo * n )
      : command( cmd )
      , next( n )
    { }

    /** get the next undo
     * @return the next undo
     */
   CanvasUndo * Next() { return next; }

  
};

#endif // CANVAS_UNDO_H

